@@ -4,7 +4,6 @@
 # This work is licensed under the GNU GPLv2 or later.
 # See the COPYING file in the top-level directory.
 
-import logging
 import os
 import threading
 import time
@@ -13,23 +12,18 @@ import traceback
 import libvirt
 
 import virtinst
+from virtinst import log
 from virtinst import pollhelpers
-from virtinst import util
 
-from . import connectauth
+from .lib import connectauth
 from .baseclass import vmmGObject
-from .domain import vmmDomain
-from .interface import vmmInterface
-from .libvirtenummap import LibvirtEnumMap
-from .network import vmmNetwork
-from .nodedev import vmmNodeDevice
-from .statsmanager import vmmStatsManager
-from .storagepool import vmmStoragePool
-
-
-# debugging helper to turn off events
-# Can be enabled with virt-manager --test-no-events
-FORCE_DISABLE_EVENTS = False
+from .lib.libvirtenummap import LibvirtEnumMap
+from .object.domain import vmmDomain
+from .object.interface import vmmInterface
+from .object.network import vmmNetwork
+from .object.nodedev import vmmNodeDevice
+from .object.storagepool import vmmStoragePool
+from .lib.statsmanager import vmmStatsManager
 
 
 class _ObjectList(vmmGObject):
@@ -291,7 +285,7 @@ class vmmConnection(vmmGObject):
                     try:
                         ret.append(vol.get_xmlobj(refresh_if_nec=False))
                     except Exception as e:
-                        logging.debug("Fetching volume XML failed: %s", e)
+                        log.debug("Fetching volume XML failed: %s", e)
             return ret
         self._backend.cb_fetch_all_vols = fetch_all_vols
 
@@ -361,7 +355,7 @@ class vmmConnection(vmmGObject):
 
     def get_cache_dir(self):
         uri = self.get_uri().replace("/", "_")
-        ret = os.path.join(util.get_cache_dir(), uri)
+        ret = os.path.join(self._backend.get_app_cache_dir(), uri)
         if not os.path.exists(ret):
             os.makedirs(ret, 0o755)
         return ret
@@ -375,7 +369,7 @@ class vmmConnection(vmmGObject):
         if fmt != "qcow2":
             return fmt
 
-        if self.check_support(self._backend.SUPPORT_CONN_DEFAULT_QCOW2):
+        if self.support.conn_default_qcow2():
             return fmt
         return None
 
@@ -431,49 +425,40 @@ class vmmConnection(vmmGObject):
     # API support helpers #
     #######################
 
-    for _supportname in [_supportname for _supportname in
-                         dir(virtinst.VirtinstConnection) if
-                         _supportname.startswith("SUPPORT_")]:
-        locals()[_supportname] = getattr(virtinst.VirtinstConnection,
-                                         _supportname)
-    def check_support(self, *args):
-        # pylint: disable=no-value-for-parameter
-        return self._backend.check_support(*args)
+    @property
+    def support(self):
+        return self._backend.support
 
     def is_storage_capable(self):
         if self._storage_capable is None:
-            self._storage_capable = self.check_support(
-                                        self._backend.SUPPORT_CONN_STORAGE)
+            self._storage_capable = self.support.conn_storage()
             if self._storage_capable is False:
-                logging.debug("Connection doesn't seem to support storage "
+                log.debug("Connection doesn't seem to support storage "
                               "APIs. Skipping all storage polling.")
 
         return self._storage_capable
 
     def is_network_capable(self):
         if self._network_capable is None:
-            self._network_capable = self.check_support(
-                                       self._backend.SUPPORT_CONN_NETWORK)
+            self._network_capable = self.support.conn_network()
             if self._network_capable is False:
-                logging.debug("Connection doesn't seem to support network "
+                log.debug("Connection doesn't seem to support network "
                               "APIs. Skipping all network polling.")
 
         return self._network_capable
 
     def is_interface_capable(self):
         if self._interface_capable is None:
-            self._interface_capable = self.check_support(
-                                       self._backend.SUPPORT_CONN_INTERFACE)
+            self._interface_capable = self.support.conn_interface()
             if self._interface_capable is False:
-                logging.debug("Connection doesn't seem to support interface "
+                log.debug("Connection doesn't seem to support interface "
                               "APIs. Skipping all interface polling.")
 
         return self._interface_capable
 
     def is_nodedev_capable(self):
         if self._nodedev_capable is None:
-            self._nodedev_capable = self.check_support(
-                                            self._backend.SUPPORT_CONN_NODEDEV)
+            self._nodedev_capable = self.support.conn_nodedev()
         return self._nodedev_capable
 
     def _get_flags_helper(self, obj, key, check_func):
@@ -501,44 +486,27 @@ class vmmConnection(vmmGObject):
             act   = 0
             inact = 0
 
-            if self.check_support(
-                    self._backend.SUPPORT_DOMAIN_XML_INACTIVE, vm):
+            if self.support.domain_xml_inactive(vm):
                 inact = libvirt.VIR_DOMAIN_XML_INACTIVE
             else:
-                logging.debug("Domain XML inactive flag not supported.")
+                log.debug("Domain XML inactive flag not supported.")
 
-            if self.check_support(
-                    self._backend.SUPPORT_DOMAIN_XML_SECURE, vm):
+            if self.support.domain_xml_secure(vm):
                 inact |= libvirt.VIR_DOMAIN_XML_SECURE
                 act = libvirt.VIR_DOMAIN_XML_SECURE
             else:
-                logging.debug("Domain XML secure flag not supported.")
+                log.debug("Domain XML secure flag not supported.")
 
             return inact, act
 
         return self._get_flags_helper(vm, key, check_func)
 
-    def get_interface_flags(self, iface):
-        key = "interface"
-
-        def check_func():
-            act   = 0
-            inact = 0
-
-            if self.check_support(
-                    self._backend.SUPPORT_INTERFACE_XML_INACTIVE, iface):
-                inact = libvirt.VIR_INTERFACE_XML_INACTIVE
-            else:
-                logging.debug("Interface XML inactive flag not supported.")
-
-            return (inact, act)
-
-        return self._get_flags_helper(iface, key, check_func)
-
     def get_default_pool(self):
-        for p in self.list_pools():
-            if p.get_name() == "default":
-                return p
+        poolxml = virtinst.StoragePool.lookup_default_pool(self.get_backend())
+        if poolxml:
+            for p in self.list_pools():
+                if p.get_name() == poolxml.name:
+                    return p
         return None
 
     def get_vol_by_path(self, path):
@@ -549,7 +517,7 @@ class vmmConnection(vmmGObject):
                         return vol
                 except Exception as e:
                     # Errors can happen if the volume disappeared, bug 1092739
-                    logging.debug("Error looking up volume from path=%s: %s",
+                    log.debug("Error looking up volume from path=%s: %s",
                         path, e)
         return None
 
@@ -561,7 +529,7 @@ class vmmConnection(vmmGObject):
     def _change_state(self, newstate):
         if self._state != newstate:
             self._state = newstate
-            logging.debug("conn=%s changed to state=%s",
+            log.debug("conn=%s changed to state=%s",
                 self.get_uri(), self.get_state_text())
             self.emit("state-changed")
 
@@ -617,7 +585,7 @@ class vmmConnection(vmmGObject):
     # nodedev helper functions #
     ############################
 
-    def filter_nodedevs(self, devtype=None, devcap=None):
+    def filter_nodedevs(self, devtype):
         retdevs = []
         for dev in self.list_nodedevs():
             try:
@@ -626,36 +594,15 @@ class vmmConnection(vmmGObject):
                 # Libvirt nodedev XML fetching can be busted
                 # https://bugzilla.redhat.com/show_bug.cgi?id=1225771
                 if e.get_error_code() != libvirt.VIR_ERR_NO_NODE_DEVICE:
-                    logging.debug("Error fetching nodedev XML", exc_info=True)
+                    log.debug("Error fetching nodedev XML", exc_info=True)
                 continue
 
             if devtype and xmlobj.device_type != devtype:
                 continue
 
-            if devcap:
-                if (not hasattr(xmlobj, "capability_type") or
-                    xmlobj.capability_type != devcap):
-                    continue
-
             retdevs.append(dev)
-
         return retdevs
 
-    def get_nodedev_count(self, devtype, vendor, product):
-        count = 0
-        devs = self.filter_nodedevs(devtype)
-
-        for dev in devs:
-            if (vendor == dev.xmlobj.vendor_id and
-                product == dev.xmlobj.product_id):
-                count += 1
-
-        logging.debug("There are %d node devices with "
-                      "vendorId: %s, productId: %s",
-                       count, vendor, product)
-
-        return count
-
 
     ###################################
     # Libvirt object creation methods #
@@ -667,8 +614,6 @@ class vmmConnection(vmmGObject):
         return self._backend.networkDefineXML(xml)
     def define_pool(self, xml):
         return self._backend.storagePoolDefineXML(xml, 0)
-    def define_interface(self, xml):
-        return self._backend.interfaceDefineXML(xml, 0)
 
     def rename_object(self, obj, origxml, newxml, oldconnkey):
         if obj.is_domain():
@@ -690,11 +635,11 @@ class vmmConnection(vmmGObject):
             newobj = define_cb(newxml)
         except Exception as renameerr:
             try:
-                logging.debug("Error defining new name %s XML",
+                log.debug("Error defining new name %s XML",
                     obj.class_name(), exc_info=True)
                 newobj = define_cb(origxml)
             except Exception as fixerr:
-                logging.debug("Failed to redefine original %s!",
+                log.debug("Failed to redefine original %s!",
                     obj.class_name(), exc_info=True)
                 raise RuntimeError(
                     _("%s rename failed. Attempting to recover also "
@@ -728,7 +673,7 @@ class vmmConnection(vmmGObject):
         eventstr = args.pop(-1)
 
         name = domain.name()
-        logging.debug("domain xmlmisc event: domain=%s event=%s args=%s",
+        log.debug("domain xmlmisc event: domain=%s event=%s args=%s",
                 name, eventstr, args)
         obj = self.get_vm(name)
         if not obj:
@@ -741,7 +686,7 @@ class vmmConnection(vmmGObject):
         ignore = userdata
 
         name = domain.name()
-        logging.debug("domain lifecycle event: domain=%s %s", name,
+        log.debug("domain lifecycle event: domain=%s %s", name,
                 LibvirtEnumMap.domain_lifecycle_str(state, reason))
 
         obj = self.get_vm(name)
@@ -756,7 +701,7 @@ class vmmConnection(vmmGObject):
         ignore = userdata
 
         name = domain.name()
-        logging.debug("domain agent lifecycle event: domain=%s %s", name,
+        log.debug("domain agent lifecycle event: domain=%s %s", name,
                 LibvirtEnumMap.domain_agent_lifecycle_str(state, reason))
 
         obj = self.get_vm(name)
@@ -771,7 +716,7 @@ class vmmConnection(vmmGObject):
         ignore = userdata
 
         name = network.name()
-        logging.debug("network lifecycle event: network=%s %s",
+        log.debug("network lifecycle event: network=%s %s",
                 name, LibvirtEnumMap.network_lifecycle_str(state, reason))
         obj = self.get_net(name)
 
@@ -786,7 +731,7 @@ class vmmConnection(vmmGObject):
         ignore = userdata
 
         name = pool.name()
-        logging.debug("storage pool lifecycle event: pool=%s %s",
+        log.debug("storage pool lifecycle event: pool=%s %s",
             name, LibvirtEnumMap.storage_lifecycle_str(state, reason))
 
         obj = self.get_pool(name)
@@ -801,7 +746,7 @@ class vmmConnection(vmmGObject):
         ignore = userdata
 
         name = pool.name()
-        logging.debug("storage pool refresh event: pool=%s", name)
+        log.debug("storage pool refresh event: pool=%s", name)
 
         obj = self.get_pool(name)
 
@@ -816,7 +761,7 @@ class vmmConnection(vmmGObject):
         ignore = userdata
 
         name = dev.name()
-        logging.debug("node device lifecycle event: nodedev=%s %s",
+        log.debug("node device lifecycle event: nodedev=%s %s",
             name, LibvirtEnumMap.nodedev_lifecycle_str(state, reason))
 
         self.schedule_priority_tick(pollnodedev=True, force=True)
@@ -826,7 +771,7 @@ class vmmConnection(vmmGObject):
         ignore = userdata
 
         name = dev.name()
-        logging.debug("node device update event: nodedev=%s", name)
+        log.debug("node device update event: nodedev=%s", name)
 
         obj = self.get_nodedev(name)
 
@@ -834,23 +779,25 @@ class vmmConnection(vmmGObject):
             self.idle_add(obj.recache_from_event_loop)
 
     def _add_conn_events(self):
-        if not self.check_support(
-                self._backend.SUPPORT_CONN_WORKING_XEN_EVENTS):
+        if not self.support.conn_working_xen_events():
             return
 
+        def _check_events_disabled():
+            if self.config.CLITestOptions.no_events:
+                raise RuntimeError("events disabled via cli")
+
         try:
-            if FORCE_DISABLE_EVENTS:
-                raise RuntimeError("FORCE_DISABLE_EVENTS = True")
+            _check_events_disabled()
 
             self._domain_cb_ids.append(
                 self.get_backend().domainEventRegisterAny(
                 None, libvirt.VIR_DOMAIN_EVENT_ID_LIFECYCLE,
                 self._domain_lifecycle_event, None))
             self.using_domain_events = True
-            logging.debug("Using domain events")
+            log.debug("Using domain events")
         except Exception as e:
             self.using_domain_events = False
-            logging.debug("Error registering domain events: %s", e)
+            log.debug("Error registering domain events: %s", e)
 
         def _add_domain_xml_event(eventname, eventval, cb=None):
             if not self.using_domain_events:
@@ -863,7 +810,7 @@ class vmmConnection(vmmGObject):
                     self.get_backend().domainEventRegisterAny(
                     None, eventid, cb, eventname))
             except Exception as e:
-                logging.debug("Error registering %s event: %s",
+                log.debug("Error registering %s event: %s",
                     eventname, e)
 
         _add_domain_xml_event("VIR_DOMAIN_EVENT_ID_BALLOON_CHANGE", 13)
@@ -874,22 +821,20 @@ class vmmConnection(vmmGObject):
                               self._domain_agent_lifecycle_event)
 
         try:
-            if FORCE_DISABLE_EVENTS:
-                raise RuntimeError("FORCE_DISABLE_EVENTS = True")
+            _check_events_disabled()
 
             eventid = getattr(libvirt, "VIR_NETWORK_EVENT_ID_LIFECYCLE", 0)
             self._network_cb_ids.append(
                 self.get_backend().networkEventRegisterAny(
                 None, eventid, self._network_lifecycle_event, None))
             self.using_network_events = True
-            logging.debug("Using network events")
+            log.debug("Using network events")
         except Exception as e:
             self.using_network_events = False
-            logging.debug("Error registering network events: %s", e)
+            log.debug("Error registering network events: %s", e)
 
         try:
-            if FORCE_DISABLE_EVENTS:
-                raise RuntimeError("FORCE_DISABLE_EVENTS = True")
+            _check_events_disabled()
 
             eventid = getattr(libvirt,
                               "VIR_STORAGE_POOL_EVENT_ID_LIFECYCLE", 0)
@@ -902,14 +847,13 @@ class vmmConnection(vmmGObject):
                 self.get_backend().storagePoolEventRegisterAny(
                 None, refreshid, self._storage_pool_refresh_event, None))
             self.using_storage_pool_events = True
-            logging.debug("Using storage pool events")
+            log.debug("Using storage pool events")
         except Exception as e:
             self.using_storage_pool_events = False
-            logging.debug("Error registering storage pool events: %s", e)
+            log.debug("Error registering storage pool events: %s", e)
 
         try:
-            if FORCE_DISABLE_EVENTS:
-                raise RuntimeError("FORCE_DISABLE_EVENTS = True")
+            _check_events_disabled()
 
             eventid = getattr(libvirt, "VIR_NODE_DEVICE_EVENT_ID_LIFECYCLE", 0)
             updateid = getattr(libvirt, "VIR_NODE_DEVICE_EVENT_ID_UPDATE", 1)
@@ -921,10 +865,10 @@ class vmmConnection(vmmGObject):
                 None, updateid, self._node_device_update_event, None))
 
             self.using_node_device_events = True
-            logging.debug("Using node device events")
+            log.debug("Using node device events")
         except Exception as e:
             self.using_network_events = False
-            logging.debug("Error registering node device events: %s", e)
+            log.debug("Error registering node device events: %s", e)
 
 
     ######################################
@@ -937,7 +881,7 @@ class vmmConnection(vmmGObject):
 
     def close(self):
         if not self.is_disconnected():
-            logging.debug("conn.close() uri=%s", self.get_uri())
+            log.debug("conn.close() uri=%s", self.get_uri())
         self._closing = True
 
         try:
@@ -951,7 +895,7 @@ class vmmConnection(vmmGObject):
                 for eid in self._node_device_cb_ids:
                     self._backend.nodeDeviceEventDeregisterAny(eid)
         except Exception:
-            logging.debug("Failed to deregister events in conn cleanup",
+            log.debug("Failed to deregister events in conn cleanup",
                 exc_info=True)
         finally:
             self._domain_cb_ids = []
@@ -970,13 +914,13 @@ class vmmConnection(vmmGObject):
                 self._remove_object_signal(obj)
                 obj.cleanup()
             except Exception as e:
-                logging.debug("Failed to cleanup %s: %s", obj, e)
+                log.debug("Failed to cleanup %s: %s", obj, e)
         self._objects.cleanup()
         self._objects = _ObjectList()
 
         closeret = self._backend.close()
-        if closeret == 1 and self.config.test_leak_debug:
-            logging.debug("LEAK: conn close() returned 1, "
+        if closeret == 1 and self.config.CLITestOptions.leak_debug:
+            log.debug("LEAK: conn close() returned 1, "
                     "meaning refs may have leaked.")
 
         self._change_state(self._STATE_DISCONNECTED)
@@ -998,7 +942,7 @@ class vmmConnection(vmmGObject):
 
         self._change_state(self._STATE_CONNECTING)
 
-        logging.debug("Scheduling background open thread for %s",
+        log.debug("Scheduling background open thread for %s",
                       self.get_uri())
         self._start_thread(self._open_thread, "Connect %s" % self.get_uri())
 
@@ -1021,12 +965,12 @@ class vmmConnection(vmmGObject):
 
         if (libvirt_error_code ==
             getattr(libvirt, "VIR_ERR_AUTH_CANCELLED", None)):
-            logging.debug("User cancelled auth, not raising any error.")
+            log.debug("User cancelled auth, not raising any error.")
             return False, None
 
         if (libvirt_error_code == libvirt.VIR_ERR_AUTH_FAILED and
             "not authorized" in libvirt_error_message.lower()):
-            logging.debug("Looks like we might have failed policykit "
+            log.debug("Looks like we might have failed policykit "
                           "auth. Checking to see if we have a valid "
                           "console session")
             if (not self.is_remote() and
@@ -1038,12 +982,12 @@ class vmmConnection(vmmGObject):
         return False, ConnectError
 
     def _populate_initial_state(self):
-        logging.debug("libvirt version=%s",
+        log.debug("libvirt version=%s",
                       self._backend.local_libvirt_version())
-        logging.debug("daemon version=%s",
+        log.debug("daemon version=%s",
                       self._backend.daemon_version())
-        logging.debug("conn version=%s", self._backend.conn_version())
-        logging.debug("%s capabilities:\n%s",
+        log.debug("conn version=%s", self._backend.conn_version())
+        log.debug("%s capabilities:\n%s",
                       self.get_uri(), self.caps.get_xml())
 
         # Try to create the default storage pool
@@ -1051,20 +995,17 @@ class vmmConnection(vmmGObject):
         try:
             virtinst.StoragePool.build_default_pool(self.get_backend())
         except Exception as e:
-            logging.debug("Building default pool failed: %s", str(e))
+            log.debug("Building default pool failed: %s", str(e))
 
         self._add_conn_events()
 
-        # Prime CPU cache
-        self.caps.get_cpu_values(self.caps.host.cpu.arch)
-
         try:
             self._backend.setKeepAlive(20, 1)
         except Exception as e:
             if (not isinstance(e, AttributeError) and
-                not util.is_error_nosupport(e)):
+                not self.support.is_error_nosupport(e)):
                 raise
-            logging.debug("Connection doesn't support KeepAlive, "
+            log.debug("Connection doesn't support KeepAlive, "
                 "skipping")
 
         # The initial tick will set up a threading event that will only
@@ -1135,11 +1076,11 @@ class vmmConnection(vmmGObject):
                 name = str(obj)
 
             if not self._objects.remove(obj):
-                logging.debug("Requested removal of %s=%s, but it's "
+                log.debug("Requested removal of %s=%s, but it's "
                     "not in our object list.", class_name, name)
                 continue
 
-            logging.debug("%s=%s removed", class_name, name)
+            log.debug("%s=%s removed", class_name, name)
             self._remove_object_signal(obj)
             obj.cleanup()
 
@@ -1151,24 +1092,24 @@ class vmmConnection(vmmGObject):
             class_name = obj.class_name()
 
             if initialize_failed:
-                logging.debug("Blacklisting %s=%s", class_name, obj.get_name())
+                log.debug("Blacklisting %s=%s", class_name, obj.get_name())
                 count = self._objects.add_blacklist(obj)
                 if count <= _ObjectList.BLACKLIST_COUNT:
-                    logging.debug("Object added in blacklist, count=%d", count)
+                    log.debug("Object added in blacklist, count=%d", count)
                 else:
-                    logging.debug("Object already blacklisted?")
+                    log.debug("Object already blacklisted?")
                 return
             else:
                 self._objects.remove_blacklist(obj)
 
             if not self._objects.add(obj):
-                logging.debug("New %s=%s requested, but it's already tracked.",
+                log.debug("New %s=%s requested, but it's already tracked.",
                     class_name, obj.get_name())
                 return
 
             if not obj.is_nodedev():
                 # Skip nodedev logging since it's noisy and not interesting
-                logging.debug("%s=%s status=%s added", class_name,
+                log.debug("%s=%s status=%s added", class_name,
                     obj.get_name(), obj.run_status())
             if obj.is_domain():
                 self.emit("vm-added", obj.get_connkey())
@@ -1335,13 +1276,13 @@ class vmmConnection(vmmGObject):
 
                 obj.tick(stats_update=stats_update)
             except Exception as e:
-                logging.exception("Tick for %s failed", obj)
+                log.exception("Tick for %s failed", obj)
                 if (isinstance(e, libvirt.libvirtError) and
                     (getattr(e, "get_error_code")() ==
                      libvirt.VIR_ERR_SYSTEM_ERROR)):
                     # Try a simple getInfo call to see if conn was dropped
                     self._backend.getInfo()
-                    logging.debug("vm tick raised system error but "
+                    log.debug("vm tick raised system error but "
                                   "connection doesn't seem to have dropped. "
                                   "Ignoring.")
 
@@ -1440,13 +1381,13 @@ class vmmConnection(vmmGObject):
             dom = e.get_error_domain()
             code = e.get_error_code()
 
-        logging.debug("Error polling connection %s",
+        log.debug("Error polling connection %s",
             self.get_uri(), exc_info=True)
 
         if (dom in [from_remote, from_rpc] and
             code in [sys_error, internal_error]):
             e = None
-            logging.debug("Not showing user error since libvirtd "
+            log.debug("Not showing user error since libvirtd "
                 "appears to have stopped.")
 
         self._schedule_close()
@@ -1526,3 +1467,9 @@ class vmmConnection(vmmGObject):
             *args, **kwargs)
     def _config_pretty_name_changed_cb(self):
         self.emit("state-changed")
+
+    def set_details_window_size(self, w, h):
+        self.config.set_perconn(self.get_uri(), "/window-size", (w, h))
+    def get_details_window_size(self):
+        ret = self.config.get_perconn(self.get_uri(), "/window-size")
+        return ret
